/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.common;

import junit.framework.Test;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.cosmos.rm.validation.internal.common.AbstractValidationOutput.ValidationMessage;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;

/**
 * Generated code for the test suite <b>TestMarkerOutput</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/plugin-junits/TestMarkerOutput.testsuite</i>.
 */
public class TestMarkerOutput extends HyadesTestCase
{
	/**
	 * Constructor for TestMarkerOutput.
	 * @param name
	 */
	public TestMarkerOutput(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestMarkerOutput</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testMarkerOutput = new HyadesTestSuite("TestMarkerOutput");
		testMarkerOutput.setArbiter(DefaultTestArbiter.INSTANCE).setId("E11F9FBC0D10CFA7CDBD5BD0767211DB");
		testMarkerOutput.addTest(new TestMarkerOutput("testReportMessage").setId("E11F9FBC0D10CFA7D7D10270767211DB").setTestInvocationId("E11F9FBC0D10CFA7AFD6B790767411DB"));
		return testMarkerOutput;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	/**
	 * testReportMessage
	 * @throws Exception
	 */
	public void testReportMessage() throws Exception
	{
		MarkerOutput mo = new MarkerOutput();
		
		/* Log a report, warning, and an info message */
		mo.reportError("Error Message");
		mo.reportWarning("Warning Message");
		mo.reportInfo("Info Message");
		
		/* Log a message with an invalid severity.  The message is expected to be logged as an info */
		ValidationMessage vm = new ValidationMessage();
		vm.setAttribute(ValidationMessage.ATTRIBUTE_MESSAGE, "Message with bad severity");
		vm.setAttribute(ValidationMessage.ATTRIBUTE_SEVERITY, 1000);		
		mo.reportMessage(vm);
		
		/* Query the markers and count the messages based on severity */
		IMarker[] markers = ResourcesPlugin.getWorkspace().getRoot().findMarkers(MarkerOutput.SML_MARKER_ID, false, IResource.DEPTH_INFINITE);
		int errorCount = 0, warningCount = 0, infoCount = 0;
		for (int i = 0; i < markers.length; i++)
		{
			int severity = markers[i].getAttribute(IMarker.SEVERITY, -1);
			switch (severity)
			{
				case IMarker.SEVERITY_ERROR:
					errorCount++;
					break;
				case IMarker.SEVERITY_WARNING:
					warningCount++;
					break;
				case IMarker.SEVERITY_INFO:
					infoCount++;
					break;					
			}
		}
		
		/* Make sure that there is only one error and warning message and 2 info messages */
		assertEquals("Number of error messages was expected to be 1", 1, errorCount);
		assertEquals("Number of warning messages was expected to be 1", 1, warningCount);
		assertEquals("Number of info messages was expected to be 2", 2, infoCount);
	}

}
