package org.eclipse.cosmos.rm.validation.internal.smlvalidators;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.artifacts.DocumentNode;
import org.eclipse.cosmos.rm.validation.internal.artifacts.ElementEdge;
import org.eclipse.cosmos.rm.validation.internal.artifacts.TypeNode;
import org.eclipse.cosmos.rm.validation.internal.common.SystemOutput;
import org.eclipse.cosmos.rm.validation.internal.common.UtilityTester;
import org.eclipse.cosmos.rm.validation.internal.core.IValidator;
import org.eclipse.cosmos.rm.validation.internal.databuilders.AbstractDataBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.AcyclicDataTypesList;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DataBuilderRegistry;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DocumentDOMBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.ReferenceGraphDataBuilder;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;

/**
 * Generated code for the test suite <b>TestAcyclicValidator</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestAcyclicValidator.testsuite</i>.
 */
public class TestAcyclicValidator extends HyadesTestCase
{
	/**
	 * Constructor for TestAcyclicValidator.
	 * @param name
	 */
	public TestAcyclicValidator(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestAcyclicValidator</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testAcyclicValidator = new HyadesTestSuite("TestAcyclicValidator");
		testAcyclicValidator.setArbiter(DefaultTestArbiter.INSTANCE).setId("E097D6574AC19088A59D8EB07BF211DB");
		testAcyclicValidator.addTest(new TestAcyclicValidator("testInvalidCycle").setId("EAE0EB6460661010B50F3FBA7EFE11DB").setTestInvocationId("EAE0EB6460661010C353BEC07EFE11DB"));
		testAcyclicValidator.addTest(new TestAcyclicValidator("testValidCycle1").setId("EAE0EB6460661010D3B7E9807EFE11DB").setTestInvocationId("EAE0EB6460661010795D00A07EFF11DB"));
		testAcyclicValidator.addTest(new TestAcyclicValidator("testValidCycle2").setId("EAE0EB6460661010FEE620207F0511DB").setTestInvocationId("EAE0EB64606610100FA9A8F07F0611DB"));
		return testAcyclicValidator;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	/**
	 * testInvalidCycle
	 * @throws Exception
	 */
	public void testInvalidCycle() throws Exception
	{		
		AcyclicValidator acyclicValidator = initialize(
				new TypeNode[]{
					new TypeNode("OptionalCourseRefType", "urn:university")}, 
				new String[][]{
					new String[]{"urn:university","RequiredCourseRefType"}, 
					new String[]{"urn:university","OptionalCourseRefType"}, 
					new String[]{"urn:university","OptionalCourseRefType"}});
		UtilityTester.registerInheritanceMap(new String[][]{
				new String[]{
					"urn:universityRequiredCourseRefType", 
					"urn:universityOptionalCourseRefType"}});
		
		assertFalse("The validation was expected to return false", acyclicValidator.validate());		
	}

	/** 
	 * testValidCycle1
	 * Let R1 and R2 be two reference types with sml:acyclic=”true”. Then a model is valid if it has inter-document cycles 
	 * that using instances of both R1 and R2
	 * @throws Exception
	 */
	public void testValidCycle1() throws Exception
	{
		AcyclicValidator acyclicValidator = initialize(
				new TypeNode[]{
						new TypeNode("RequiredCourseRefType", "urn:university"),
						new TypeNode("OptionalCourseRefType", "urn:university")}, 
				new String[][]{
								new String[]{"urn:university","RequiredCourseRefType"}, 
								new String[]{"urn:university","OptionalCourseRefType"}, 
								new String[]{"urn:university","RequiredCourseRefType"}});
						
		UtilityTester.registerInheritanceMap(new String[][]{});
		assertTrue("The validation was expected to return true", acyclicValidator.validate());		
	}
	
	
	private AcyclicValidator initialize(final TypeNode[] nodes, final String[][] referenceTypes)
	{
		/* The identity structure */
		UtilityTester.registerIdentityStructure(null, null, "http://interop.serviceml.org");		
		
		Map attributes = new Hashtable();
		attributes.put(IValidator.ATTRIBUTE_OUTPUT, new SystemOutput());
		AcyclicValidator acyclicValidator = new AcyclicValidator();
		acyclicValidator.initialize(attributes);
		
		DataBuilderRegistry registry = DataBuilderRegistry.instance();
		registry.registerDataStructureBuilder(ReferenceGraphDataBuilder.ID, new ReferenceGraph(referenceTypes));
		registry.registerDataStructureBuilder(AcyclicDataTypesList.ID, new AcyclicDataTypesList()
		{
			public Object getDataStructure()
			{
				final List ACYCLIC_TYPES = new ArrayList();
				for (int i = 0; i < nodes.length; i++)
				{
					ACYCLIC_TYPES.add(nodes[i]);	
				}
				
				return ACYCLIC_TYPES.iterator();
			}
		});
		
		return acyclicValidator;
	}
	
	
	/**
	 * Let R1 be a reference type with sml:acyclic=”false”. Then a model is valid
	 * if it has inter-document cycles using instances of R1
	 * 
	 * @throws Exception
	 */
	public void testValidCycle2() throws Exception
	{
		AcyclicValidator acyclicValidator = initialize(
				new TypeNode[]{}, 
				new String[][]{
						new String[]{"urn:university","OptionalCourseRefType"}, 
						new String[]{"urn:university","OptionalCourseRefType"}, 
						new String[]{"urn:university","OptionalCourseRefType"}});
		assertTrue("The validation was expected to return true", acyclicValidator.validate());		
	}
	
	
	/**
	 * Builds a dummy reference graph
	 * 
	 * @author Ali Mehregani
	 */
	private final static class ReferenceGraph extends ReferenceGraphDataBuilder
	{
		final static Map REFERENCE_GRAPH = new Hashtable();
		
		public ReferenceGraph(final String[][] referenceTypes)
		{
			final int NUM_OF_COURSES = 3;
			final String[] REFERENCE = new String[]{"Course1", "Course2", "Course0"};
			final Map aliasMap = new Hashtable();
			for (int i = 0; i < REFERENCE.length; i++)
			{
				aliasMap.put(REFERENCE[i], "aliasMap");
			}
			DataBuilderRegistry.instance().registerDataStructureBuilder(DocumentDOMBuilder.ID, new AbstractDataBuilder(){
				public Object getDataStructure()
				{			
					return aliasMap;
				}});
			
			DocumentNode[] documentNodes = new DocumentNode[NUM_OF_COURSES];
			for (int i = 0; i < NUM_OF_COURSES; i++)
			{
				DocumentNode course = new DocumentNode();
				course.addAlias("Course" + i);
				
				REFERENCE_GRAPH.put("Course" + i, course);
				documentNodes[i] = course;
			}
			
			for (int i = 0; i < NUM_OF_COURSES; i++)
			{
				ElementEdge edge = new ElementEdge(referenceTypes[i][0], referenceTypes[i][1]); 
				edge.setReference(REFERENCE[i]);
				documentNodes[i].addEdge(edge);				
			}			
		}
			
		
		public Object getDataStructure()
		{			
			return REFERENCE_GRAPH;
		}
	}
}
