/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.common;

import javax.xml.namespace.QName;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;
import javax.xml.xpath.XPathFunction;
import javax.xml.xpath.XPathFunctionResolver;

import org.eclipse.cosmos.rm.validation.internal.databuilders.DataBuilderRegistry;
import org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder;
import org.eclipse.cosmos.rm.validation.internal.reference.DrefXPathFunction;

/**
 * Provides common utility method used by multiple classes under
 * different pacakges 
 * 
 * @author Ali Mehregani
 */
public class SMLCommonUtil
{
	/**
	 * A static XPath instance used to compile XPath expressions
	 */
	public static XPath xpath;
	
	static
	{
		XPathFactory factory = XPathFactory.newInstance();
	    xpath = factory == null ? null : factory.newXPath();
	    if (xpath != null)
	    {
	    	/* Register the dref XPath function extension */
	    	xpath.setXPathFunctionResolver(new XPathFunctionResolver()
	    	{
	    		/**
	    		 * @see XPathFunctionResolver#resolveFunction(QName, int)
	    		 */
				public XPathFunction resolveFunction(QName functionName, int arity)
				{
					if (ISMLConstants.SML_FN_URI == functionName.getNamespaceURI() && arity == DrefXPathFunction.ARITY)
						return DrefXPathFunction.instance();
					return null;
				}
			});
	    }
	}
	
	/**
	 * A helper method provided to retrieve the data structure for a databuilder
	 * with the given id.
	 * 
	 * @param id The data builder id
	 */
	public static Object retrieveDataStructure(String databuilderId)
	{		
		IDataBuilder databuilder = DataBuilderRegistry.instance().getDataStructureBuilder(databuilderId);
		if (databuilder == null)
			return null;
		
		return databuilder.getDataStructure();
	}

	
	/**
	 * Removes line breaks and any spaces that follow a line break
	 * 
	 * @param str The string
	 * @return str with line break and white spaces that follow line breaks
	 * removed. 
	 */
	public static String removeLineBreaks(String str)
	{
		if (str == null)
			return null;
		
		StringBuffer sb = new StringBuffer();
		char currentChar, lastChar = 0;
		for (int i = 0, strLength = str.length(); i < strLength; i++)
		{
			currentChar = str.charAt(i);
				
			if (currentChar == '\r' || (lastChar == '\n' && (currentChar == '\t' || currentChar == ' ')))
			{
				continue;
			}
			
			if (currentChar != '\n')
			{
				sb.append(currentChar);				
			}
			lastChar = currentChar;
		}
		
		return sb.toString();
	}
}
